/**
  *  \file gfx/gfxalpha.cc
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "gfx/gfxalpha.h"

GfxAlphaFilter::GfxAlphaFilter(uint8 f_alpha, GfxCanvas& can)
    : GfxFilter(can), f_alpha(f_alpha)
{ }

GfxAlphaFilter::~GfxAlphaFilter()
{ }

static inline int
multAlpha(uint8 a, uint8 b)
{
    return (a * b) / 255;
}

void
GfxAlphaFilter::drawHLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha)
{
    parent.drawHLine(pt, npix, color, pat, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::drawVLine(const GfxPoint& pt, int npix, uint32 color, uint8 pat, alpha_type alpha)
{
    parent.drawVLine(pt, npix, color, pat, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::drawPixel(const GfxPoint& pt, uint32 color, alpha_type alpha)
{
    parent.drawPixel(pt, color, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::drawBar(GfxRect rect, uint32 color, uint32 bg, const GfxFillPattern& pat, alpha_type alpha)
{
    parent.drawBar(rect, color, bg, pat, multAlpha(alpha, f_alpha));
}

void
GfxAlphaFilter::blitSurface(const GfxPoint& pt, GfxCanvas& src, GfxRect rect)
{
    // FIXME: alpha
    parent.blitSurface(pt, src, rect);
}

void
GfxAlphaFilter::blitPattern(GfxRect rect, const GfxPoint& pt, int bytes_per_line, const uint8* data, uint32 color, uint32 bg, alpha_type alpha)
{
    parent.blitPattern(rect, pt, bytes_per_line, data, color, bg, multAlpha(alpha, f_alpha));
}

GfxRect
GfxAlphaFilter::computeClipRect(GfxRect r)
{
    return parent.computeClipRect(r);
}

bool
GfxAlphaFilter::computeOffset(GfxCanvas& other, GfxPoint& pt)
{
    return this == &other
        || parent.computeOffset(other, pt);
}

bool
GfxAlphaFilter::isVisible(GfxRect r)
{
    return parent.isVisible(r);
}

bool
GfxAlphaFilter::isClipped(GfxRect r)
{
    return parent.isClipped(r);
}
