/**
  *  \file int/if/ionif.h
  *  \brief Interpreter Game Interface: Ion Storms
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_IONIF_H
#define PCC_V2_INT_IF_IONIF_H

#include "int/context.h"
#include "cpluslib/types.h"
#include "int/if/objcontext.h"

class IntValue;
class IntArgBlock;
class GIonStorm;

IntValue*   IFIonGet(IntArgBlock& args);
IntContext* IFIonMake();
int32_t     IFIonDim(int32_t dim);

/** Ion Storm context. */
class IntIonContext : public IntObjectContext {
 public:
    IntIonContext(int iid);
    ~IntIonContext();

    // IntContext:
    int32_t   lookup(const string_t& name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    IntContext* clone();
    string_t    toString(bool readable);
    void        store(IntTagNode& sv, Stream& aux);
    void        enumProperties(IntPropertyAcceptor& acceptor);
};

/** Ion Storm property identifier. */
enum IntIonProperty {
    iipClass,
    iipHeadingInt,
    iipHeadingName,
    iipId,
    iipLocX,
    iipLocY,
    iipMarked,
    iipName,
    iipRadius,
    iipSpeedInt,
    iipSpeedName,
    iipStatusFlag,
    iipStatusName,
    iipVoltage
};

IntValue* getIonProperty(const GIonStorm& ion, IntIonProperty iip);
void      setIonProperty(GIonStorm& ion, IntIonProperty iip, IntValue* value);

#endif
