/**
  *  \file ui/widgets/keytranslator.cc
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widgets/keytranslator.h"

/** Constructor. Makes a blank UIKeyTranslator which doesn't yet translate any keys. */
UIKeyTranslator::UIKeyTranslator()
    : UIFixedWidget()
{ }

/** Initialize from static table.
    \param keys points to \c n Init structures;
    \param n     number of initial mappings. */
UIKeyTranslator::UIKeyTranslator(const Item keys[], unsigned n)
    : UIFixedWidget()
{
    add(keys, n);
}

bool
UIKeyTranslator::handleEvent(const UIEvent& e, bool second_pass)
{
    if (second_pass && e.type == ev_Key) {
        if (Item* i = find(e.key.code)) {
            setSelectedWidget();
            if (i->is_stop)
                stop(i->command);
            else
                handleCommand(UICommand(i->command, e.key.arg), *this);
            return true;
        }
    }
    return UIWidget::handleEvent(e, second_pass);
}

void
UIKeyTranslator::drawContent(GfxCanvas&)
{ }

/** Add range of key translations. */
UIKeyTranslator&
UIKeyTranslator::add(const Item keys[], unsigned n)
{
    for (unsigned i = 0; i < n; ++i) {
        if (Item* p = find(keys[i].key))
            *p = keys[i];
        else
            this->keys.push_back(keys[i]);
    }
    return *this;
}

/** Add a key translation. */
UIKeyTranslator&
UIKeyTranslator::add(UIKey key, int command, bool is_stop)
{
    Item i;
    i.key     = key;
    i.command = command;
    i.is_stop = is_stop;
    return add(&i, 1);
}

UIKeyTranslator::Item*
UIKeyTranslator::find(UIKey key)
{
    for (unsigned i = 0; i < keys.size(); ++i)
        if (keys[i].key == key)
            return &keys[i];
    return 0;
}

